class Proxy6Error(Exception):
    """Proxy6 API error"""

    def __init__(self, *, code: int = None, description: str = None):
        if code is not None:
            self.code = code
        if description is not None:
            self.description = description

        if self.__class__ != Proxy6Error:
            super().__init__(self.__class__.__doc__)
        else:
            super().__init__(f"{description} (code {code})")


class CountError(Proxy6Error):
    """Wrong proxies quantity, wrong amount or no quantity input"""

    code = 200
    description = "Error count"


class NoMoneyError(Proxy6Error):
    """Balance error. Zero or low balance on your account"""

    code = 400
    description = "Error no money"


def select(data: dict) -> Proxy6Error:
    code = data.pop('error_id')
    description = data.pop('error')

    for Error in (CountError, NoMoneyError):
        if code == Error.code:
            assert description == Error.description
            return Error

    return Proxy6Error(code=code, description=description)



import enum
import ipaddress

from typing import Sequence

from marshmallow import EXCLUDE, fields, post_load, pre_load, Schema

from . import types


class IPAddressField(fields.Field):
    def _serialize(self, value, attr, obj, **kwargs):
        return str(value)

    def _deserialize(self, value, attr, data, **kwargs):
        return ipaddress.ip_address(value)


class EnumField(fields.Field):
    def __init__(self, cls, *args, **kwargs):
        assert issubclass(cls, enum.Enum)
        self.enum = cls

        return super().__init__(*args, **kwargs)

    def _serialize(self, value, attr, obj, **kwargs):
        return value.value

    def _deserialize(self, value, attr, data, **kwargs):
        return self.enum(value)


class PriceInformationSchema(Schema):
    class Meta:
        unknown = EXCLUDE

    price = fields.Float(required=True)
    price_single = fields.Float(required=True)
    period = fields.Integer(required=True)
    count = fields.Integer(required=True)
    currency = fields.String(required=True)

    @post_load
    def make_obj(self, data, **kwargs):
        return types.PriceInformation(**data)


class ProxySchema(Schema):
    class Meta:
        unknown = EXCLUDE

    id = fields.Integer(required=True)

    ip = IPAddressField(required=True)

    host = fields.String(required=True)
    port = fields.Integer(required=True)

    user = fields.String(required=True)
    password = fields.String(required=True, data_key='pass')

    version = EnumField(types.ProxyVersion, required=True)
    type = EnumField(types.ProxyType, required=True)
    country = fields.String(required=True)

    purchased_at = fields.DateTime(required=True, data_key='date')
    expires_at = fields.DateTime(required=True, data_key='date_end')

    description = fields.String(required=True, data_key='descr')

    active = fields.Boolean(required=True)

    @pre_load
    def parse_version(self, data, **kwargs):
        """Convert version from string to integer"""
        data['version'] = int(data['version'])
        return data

    @post_load
    def make_proxy(self, data, **kwargs):
        return types.Proxy(**data)


class PurchaseSchema(PriceInformationSchema):
    proxies = fields.Nested(ProxySchema, many=True, required=True, data_key='list')

    @pre_load
    def set_country_and_description(self, data, **kwargs):
        country = data.pop('country')
        description = data.pop('description')

        for proxy in data['list']:
            proxy.update({'country': country, 'descr': description})

        return data

    @post_load
    def make_obj(self, data, **kwargs):
        return types.Purchase(**data)


class ProlongationSchema(PriceInformationSchema):
    def __init__(self, existing_proxies: Sequence[types.Proxy], *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.existing_proxies = existing_proxies

    proxies = fields.Nested(ProxySchema, many=True, required=True, data_key='list')

    @pre_load
    def use_existing_proxies(self, data, **kwargs):
        temp = data.pop('list')
        assert len(temp) <= len(self.existing_proxies)

        used_proxies = (
            proxy for proxy in self.existing_proxies if str(proxy.id) in temp
        )

        data['list'] = [
            {**ProxySchema().dump(proxy), 'date_end': temp[str(proxy.id)]['date_end']}
            for proxy in used_proxies
        ]

        return data

    @post_load
    def make_obj(self, data, **kwargs):
        return types.Prolongation(**data)



from __future__ import annotations

import datetime
import enum
import ipaddress

from dataclasses import dataclass, field
from decimal import Decimal
from typing import Sequence, Union


@dataclass
class Account:
    user_id: int
    balance: Decimal
    currency: str


@dataclass
class PriceInformation:
    price: float
    price_single: float
    period: int
    count: int
    currency: str


@dataclass
class Purchase(PriceInformation):
    proxies: Sequence[Proxy]


Prolongation = Purchase


class ProxyState(enum.Enum):
    ALL = 'all'
    ACTIVE = 'active'
    NOT_ACTIVE = 'expiring'
    EXPIRED = 'expired'


class ProxyType(enum.Enum):
    HTTP = 'http'
    SOCKS5 = 'socks'


class ProxyVersion(enum.IntEnum):
    IPv4 = 4
    IPv4_SHARED = 3
    IPv6 = 6


@dataclass
class Proxy:
    id: int

    ip: Union[ipaddress.IPv4Address, ipaddress.IPv6Address]

    host: str
    port: int

    user: str
    password: str

    version: ProxyVersion
    type: ProxyType
    country: str

    purchased_at: datetime.datetime
    expires_at: datetime.datetime

    active: bool
    description: str = field(default="")

    @property
    def url(self):
        scheme = self.type.name.lower()
        return f'{scheme}://{self.user}:{self.password}@{self.host}:{self.port}'



import copy
import enum

from decimal import Decimal
from typing import Iterable, List, Optional, Sequence
from urllib.parse import urljoin

import requests

from . import errors, schemas
from .types import (
    Account,
    Purchase,
    PriceInformation,
    Prolongation,
    Proxy,
    ProxyState,
    ProxyType,
    ProxyVersion,
)


def _clean_params(**kwargs) -> dict:
    return {
        k: (v.value if isinstance(v, enum.Enum) else v)
        for k, v in kwargs.items()
        if v is not None
    }


def _format_list_param(items: list) -> str:
    """Format list for Proxy6's non-standard GET query parameters format"""
    return ','.join(map(str, items))


class Proxy6:
    def __init__(self, api_key: str):
        self._base_url = f'https://proxy6.net/api/{api_key}/'
        self._session = requests.Session()

    def _request(self, method: str, *, params: Optional[dict] = None) -> dict:
        url = urljoin(self._base_url, method)
        response = self._session.get(url, params=params)

        assert response.ok  # TODO: handle other cases

        data = response.json()
        if data.pop('status') != 'yes':
            raise errors.select(data)

        return data

    @staticmethod
    def _pop_common_fields(data: dict):
        for key in ('user_id', 'balance', 'currency'):
            del data[key]

    def get_account(self) -> Account:
        """
        Get account information

        :returns: account information

        :raises Proxy6Error:
        """
        data = self._request('getcountry')
        return Account(
            user_id=int(data['user_id']),
            balance=Decimal(data['balance']),
            currency=data['currency'],
        )

    def get_price(
        self, *, count: int, period: int, version: Optional[ProxyVersion] = None
    ) -> PriceInformation:
        """
        Used to get information about the cost of the order, depending on the
        period and number of proxies

        :param count: Number of proxies
        :param period: Number of days
        :param version: Proxy version (default is IPv6)

        :returns: price data

        :raises Proxy6Error:
        """
        params = _clean_params(count=count, period=period, version=version)
        data = self._request('getprice', params=params)

        return schemas.PriceInformationSchema().load(data)

    def get_count(self, *, country: str, version: Optional[ProxyVersion] = None) -> int:
        """
        Get information about the amount of proxies available to purchase for
        a selected country

        :param count: Country code in ISO2 format
        :param version: Proxy version (default is IPv6)

        :returns: available amount of proxies

        :raises Proxy6Error:
        """
        params = _clean_params(country=country, version=version)
        data = self._request('getcount', params=params)

        self.__class__._pop_common_fields(data)
        assert len(data) == 1

        return data['count']

    def get_countries(self, *, version: Optional[ProxyVersion] = None) -> List[str]:
        """
        Get information on available for proxies purchase countries

        :param version: Proxy version (default is IPv6)

        :returns: list of country codes in ISO2 format

        :raises Proxy6Error:
        """
        params = _clean_params(version=version)
        data = self._request('getcountry', params=params)

        self.__class__._pop_common_fields(data)
        assert len(data) == 1

        return data['list']

    def get_proxies(
        self, *, state: Optional[ProxyState] = None, description: Optional[str] = None
    ) -> Sequence[Proxy]:
        """
        Get the list of proxies

        :param state: filter proxies by state
        :param description: filter proxies by technical comment

        :returns: list of proxies

        :raises Proxy6Error:
        """
        params = _clean_params(state=state, descr=description, nokey=True)
        data = self._request('getproxy', params=params)

        self.__class__._pop_common_fields(data)

        proxies = schemas.ProxySchema().load(data['list'], many=True)

        assert len(proxies) == data['list_count']
        if description is not None:
            assert all(proxy.description == description for proxy in proxies)

        return proxies

    def set_type(self, *, proxies: Iterable[Proxy], type: ProxyType) -> None:
        """
        Change the protocol type of proxies

        :param proxies: proxies to set the protocol for
        :param type: new proxy type

        :raises Proxy6Error:
        """
        params = _clean_params(
            ids=_format_list_param(proxy.id for proxy in proxies), type=type
        )
        self._request('settype', params=params)

    def set_description(
        self,
        *,
        new: str,
        old: Optional[str] = None,
        proxies: Optional[Iterable[Proxy]] = None,
    ) -> int:
        """
        Update technical comments in the proxy list that was added when buying

        :param proxies: proxies to set the description for
        :param new: new description
        :param old: technical comments to be changed, maximum 50 characters

        :returns: amount of proxies that wer changed

        :raises Proxy6Error:
        """
        assert old is None or len(old) <= 50

        params = _clean_params(
            new=new, old=old, ids=_format_list_param(proxy.id for proxy in proxies)
        )
        return self._request('setdescr', params=params).pop('count')

    def buy(
        self,
        *,
        count: int,
        period: int,
        country: str,
        version: Optional[ProxyVersion] = None,
        type: Optional[ProxyType] = None,
        description: str = "",
        auto_renew: bool = False,
    ) -> Purchase:
        """
        Buy proxies

        :param count: amount of proxies to purchase
        :param period: period for which proxies are purchased in days
        :param country: country in ISO2 format
        :param version: proxy version, defaults to IPv6
        :param type: proxy protocol
        :param description: technical comment for proxies list, max 50 characters
        :param auto_renew: enable auto-renewal for purchased proxies

        :returns: purchase information

        :raises Proxy6Error:
        """
        assert len(description) <= 50

        params = _clean_params(
            count=count,
            period=period,
            country=country,
            version=version,
            type=type,
            descr=description or None,
            auto_prolong=auto_renew or None,
            nokey=True,
        )
        return schemas.PurchaseSchema().load(
            {'description': description or "", **self._request('buy', params=params)}
        )

    def prolong(self, *, period: int, proxies: Iterable[Proxy]) -> Prolongation:
        """
        Extend existing proxies period

        :param period: extension of the period in days
        :param proxies: proxies to prolong

        :returns: prolongation information

        :raises Proxy6Error:
        """
        proxies = copy.deepcopy(proxies)

        params = _clean_params(
            period=period, ids=_format_list_param(proxy.id for proxy in proxies)
        )
        return schemas.ProlongationSchema(proxies).load(
            self._request('prolong', params=params)
        )

    def delete(self, *, proxies: Iterable[Proxy]) -> int:
        """
        Delete proxies

        :param proxies: proxies to delete

        :returns: amount of proxies deleted

        :raises Proxy6Error:
        """
        params = _clean_params(ids=_format_list_param(proxy.id for proxy in proxies))
        return self._request('delete', params=params)['count']

    def delete_by_description(self, *, description: str) -> int:
        """
        Delete proxies having the given description

        :param description: description to select proxies with

        :returns: amount of proxies deleted

        :raises Proxy6Error:
        """
        params = _clean_params(descr=description)
        return self._request('delete', params=params)['count']

    def is_proxy_valid(self, *, proxy_id: int) -> bool:
        """
        Checks the validity of a proxy

        :param proxy_id: proxy identifier

        :returns: proxy validity status

        :raises Proxy6Error:
        """
        params = _clean_params(ids=proxy_id)
        data = self._request('check', params=params)

        self.__class__._pop_common_fields(data)

        assert data.pop('proxy_id') == proxy_id
        assert len(data) == 1

        return data['proxy_status']



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['proxy6']

package_data = \
{'': ['*']}

install_requires = \
['marshmallow==3.0.0rc9', 'requests>=2.22,<3.0']

setup_kwargs = {
    'name': 'proxysix',
    'version': '1.0.0a4',
    'description': 'API wrapper for Proxy6',
    'long_description': '# Proxy6 API wrapper\n[![Build status](https://api.travis-ci.com/michaeldel/proxy6.svg?branch=master)](https://travis-ci.com/michaeldel/proxy6)\n[![PyPI - Status](https://img.shields.io/pypi/v/proxysix)](https://pypi.org/project/proxysix/)\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/proxysix)](https://pypi.org/project/proxysix/)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n## Install\nInstall the `proxysix` package with `pip`:\n```\npip install proxysix\n```\n',
    'author': 'michaeldel',
    'author_email': 'michaeldel@protonmail.com',
    'url': 'https://github.com/michaeldel/proxy6',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
